<?php
namespace Teknisa\SM\Controller;

use Teknisa\SM\Controller\SMCrud;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DataSource\DataSet;
use Teknisa\SM\Service\Sm00022000CalculoFormacaoPrecoService as CalcFormPrecService;

class Sm00022000CalculoFormacaoPrecoController extends SMCrud{
    protected $dataSourceName = 'fct_calcformprecserv_widget';
    protected $CalcFormPrecService;
     
    public function __construct ($crudService, $environment, CalcFormPrecService $CalcFormPrecService) {
        parent::__construct($crudService, $environment);
        $this->CalcFormPrecService = $CalcFormPrecService;
    }
    
    /**
     * Recebe o numero do item e verifica se ele é do contrato, da 
     * proposta ou nulo
    **/
    public function verificaNrItem($row, $tabela){
        $nrItem = null;
        if($tabela == "FCT_CALCFORMPRECSERV_ITPRO")
            $nrItem = $row["NRITEMORCAMENTO"];
        else 
            if($tabela == "FCT_CALCFORMPRECSERV_ITCONT")
                $nrItem = $row["NRITEMOCONTRATO"];
        $this->CalcFormPrecService->setVrTabelaItem($tabela, $nrItem);
        return $nrItem;
    }
    
    /**
     * A função "recebeOrdenacao" recebe a ordenação máxima para cálculo.
     */
    
    public function recebeOrdenacao(Request\Row $request, Response $response) {
        $row = $request->getRow();
        $nrFormPrecoServico = $row['NRFORMPRECOSERVICO']; 
        $tabela             = $row["TABELA"];
        
        $this->verificaNrItem($row, $tabela);
        $ordenacao    = $this->CalcFormPrecService->recebeOrdenacao($nrFormPrecoServico);
        
        $dtsOrdenacao = new DataSet('ORDENACAO', $ordenacao);
        $response->addDataSet($dtsOrdenacao);
    }   
    
    /**
     *  A função "validaIdentificador" Realiza a validação  do campo "identificador". 
     *  Não podem haver dois identificadores iguais em uma mesma composição.
     */
     
    public function validaIdentificador(Request\Row $request, Response $response) {
        $row = $request->getRow();
        $nrFormPrecoServico = $row['NRFORMPRECOSERVICO'];
        $IdItemCalculo      = $row['IDITEMCALCULO'];
        $nrCalcFormPrecServ = $row['NRCALCFORMPRECSERV'];
        $tabela             = $row["TABELA"];
        
        $this->verificaNrItem($row, $tabela);
        $identificador = $this->CalcFormPrecService->validaIdentificador($nrFormPrecoServico, $IdItemCalculo, $nrCalcFormPrecServ);
        
        $dtsIdentificador = new DataSet('IDENTIFICADOR', $identificador);
        $response->addDataSet($dtsIdentificador);
    }  
    
    /**
     *  A função "buscaIdentValores" busca os identificadores dos elementos e seus respectivos valores
     */
    
    public function buscaIdentValores(Request\Row $request, Response $response) {
        $row = $request->getRow();
        $nrFormPrecoServico = $row['NRFORMPRECOSERVICO']; 
        $tabela             = $row["TABELA"];
        $this->verificaNrItem($row, $tabela);
        $IdentValores = $this->CalcFormPrecService->buscaIdentValoresAnte($nrFormPrecoServico, $tabela);
        $dtsIdentValores = new DataSet('IDENTVAL', $IdentValores);
        $response->addDataSet($dtsIdentValores);
    }  
    
    /**
     *  A função "calcSomatorioFilhos" calcula o somatorio dos elementos filhos 
     */
     
    public function calcSomatorioFilhos(Request\Row $request, Response $response) {
        $row = $request->getRow();
        $nrCalcFormPrecServ = $row['NRCALCFORMPRECSERV'];
        $tabela             = $row["TABELA"];
        $this->verificaNrItem($row, $tabela);
        $this->CalcFormPrecService->calcSomatorioFilhos($nrCalcFormPrecServ, $tabela);
    }  
    
    /**
     *  A função "deletarPaieFilhos"  deleta os pais e os filhos.
     */
    
    public function deletarPaieFilhos(Request\Row $request, Response $response) {
        $row = $request->getRow();
        $nrCalcFormPrecServ = $row['NRCALCFORMPRECSERV'];
        $delArvore = $this->CalcFormPrecService->deletarPaieFilhos($nrCalcFormPrecServ);
        
    }  
    
     /**
     *  A função "recalculoElemento" realiza o recalculo dos elementos.
     */
    
    public function recalculoElemento(Request\Row $request, Response $response) {
        try {
            $row = $request->getRow();
            $nrCalcFormPrecServ = $row['NRCALCFORMPRECSERV'];
            $this->CalcFormPrecService->recalculoElemento($nrCalcFormPrecServ);
        }
        catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    /**
     *  A função "recalculoElemento" realiza o recalculo de toda a árvore dos elementos.
     */
    
    public function recalcularArvore(Request\Row $request, Response $response) {
        $row = $request->getRow();
        $nrformprecoservico = $row['NRFORMPRECOSERVICO'];
        $tabela             = $row["TABELA"];
        
        $this->verificaNrItem($row, $tabela);
        $this->CalcFormPrecService->recalcularArvore($nrformprecoservico);
    }
    
     /**
     *  A função "somatorioRecalculo" realiza o somatorio dos filhos para recalculo dos elementos.
     */
    
    public function somatorioRecalculo(Request\Row $request, Response $response) {
        $row = $request->getRow();
        $nrFormPrecoServico = $row['NRCALCFORMPRECSERV']; 
        $recalculo = $this->CalcFormPrecService->somatorioRecalculo($nrFormPrecoServico);
        $dtsRecalculo = new DataSet('SOMATORIO', $recalculo);
        $response->addDataSet($dtsRecalculo);
    }  
    
     /**
     *  A função "salvarRecalculo" salva o recalculo dos elementos.
     */

    public function salvarRecalculo(Request\Row $request, Response $response) {
        $row = $request->getRow();
        $nrCalcFormPrecServ = $row['NRCALCFORMPRECSERV'];
        $vrUnitario         = $row['VRUNITARIO'];
        $qtCalculo          = $row['QTCALCULO'];
        $qtmedida           = $row['QTMEDIDACALC'];
        $vrTotal            = $row['VRTOTAL'];
        
        $this->CalcFormPrecService->salvarRecalculo($nrCalcFormPrecServ, $vrUnitario, $qtCalculo, $qtmedida, $vrTotal);
        
    }  
    
    /**
     * A função "cadastramentoAutomatico realiza o cadastramento automático 
     * dos itens do cálculo da formação de preço
     */
     
    public function cadastramentoAutomatico(Request\Row $request, Response $response) {
        $row = $request->getRow();       
        $formacao      = $row["FORMACAO"];
        $idElementoPai = $row["IDELEMENTOPAI"];
        $idItemCalculo = $row["IDITEMCALCULO"];
    
        unset($row["IDITEMCALCULO"]);
        unset($row["FORMACAO"]);
        $idElemento = $row;
        
        $this->CalcFormPrecService->cadastramentoAutomatico($formacao, $idElemento, $idElementoPai, $idItemCalculo);
    }
}
?>